"use strict";
exports.__esModule = true;
var DependencyInjectionException_1 = require("./Exceptions/DependencyInjectionException");
var Container = (function () {
    function Container() {
    }
    Container.register = function (name, dependecies, clazz) {
        var dependency = new Dependency(name, dependecies, clazz);
        DependencyContainer.Instance().register(dependency);
    };
    Container.resolve = function (name) {
        var constructedDependencies = Container.constructedDependencies.filter(function (item) { return item.name === name; });
        var constructedDependency = undefined;
        if (constructedDependencies.length === 1) {
            constructedDependency = constructedDependency[0];
        }
        else {
            var service = DependencyContainer.Instance().resolve(name);
            var dependencies_1 = new Array();
            service.dependencies.forEach(function (dependency) {
                var d = Container.resolve(dependency);
                dependencies_1.push(d);
            });
            constructedDependency = service.clazz.construct(dependencies_1);
            Container.constructedDependencies.push(constructedDependency);
        }
        return constructedDependency;
    };
    Container.constructedDependencies = new Array();
    return Container;
}());
exports.Container = Container;
var Dependency = (function () {
    function Dependency(name, dependencies, clazz) {
        this.name = name;
        this.dependencies = dependencies;
        this.clazz = clazz;
    }
    return Dependency;
}());
var DependencyContainer = (function () {
    function DependencyContainer() {
        var _this = this;
        this.dependencies = new Array();
        this.filterDependencies = function (name) {
            return _this.dependencies.filter(function (item) {
                return item.name === name;
            });
        };
    }
    DependencyContainer.prototype.resolve = function (name) {
        var dependencies = this.filterDependencies(name);
        if (dependencies.length === 0) {
            throw new DependencyInjectionException_1.DependencyInjectionException("The dependency " + name + " could not be found");
        }
        else if (dependencies.length > 1) {
            throw new DependencyInjectionException_1.DependencyInjectionException("The dependency " + name + " is registred more than once");
        }
        else {
            return dependencies[0];
        }
    };
    DependencyContainer.prototype.register = function (dependency) {
        var dependencies = this.filterDependencies(dependency.name);
        if (dependencies.length === 0) {
            this.dependencies.push(dependency);
        }
        else {
            var dependencyIdx = this.dependencies.indexOf(dependencies[0]);
            this.dependencies[dependencyIdx] = dependency;
        }
    };
    DependencyContainer.Instance = function () {
        if (DependencyContainer.currentContainer) {
            return DependencyContainer.currentContainer;
        }
        return DependencyContainer.currentContainer = new DependencyContainer();
    };
    return DependencyContainer;
}());

"use strict";
exports.__esModule = true;
var DependencyInjectionException_1 = require("./Exceptions/DependencyInjectionException");
exports.DependencyInjectionException = DependencyInjectionException_1.DependencyInjectionException;
var Injectable_1 = require("./Injectable");
exports.Injectable = Injectable_1.Injectable;
var Container_1 = require("./Container");
exports.Container = Container_1.Container;

"use strict";
exports.__esModule = true;
var Injectable = (function () {
    /**
     *
     */
    function Injectable(constructor) {
        this.ctor = constructor;
    }
    /**
     * The constructor of the Injectable
     */
    Injectable.prototype.construct = function (dependencies) {
        return new this.ctor(dependencies);
    };
    return Injectable;
}());
exports.Injectable = Injectable;

"use strict";
exports.__esModule = true;
var DependencyInjectionException = (function () {
    function DependencyInjectionException(message) {
        this.name = 'DependencyInjectionException';
        this.message = message;
    }
    return DependencyInjectionException;
}());
exports.DependencyInjectionException = DependencyInjectionException;
