import { DependencyInjectionException } from './Exceptions/DependencyInjectionException';
import { Injectable } from './Injectable';

export class Container {
    private static constructedDependencies: Array<any> = new Array<any>();

    static register(name: string, dependecies: Array<string>, clazz: Injectable) {
        let dependency = new Dependency(name, dependecies, clazz);
        DependencyContainer.Instance().register(dependency);
    }

    static resolve<TType>(name: string): TType {
        let constructedDependencies = Container.constructedDependencies.filter((item) => item.name === name);
        let constructedDependency = undefined;
        if (constructedDependencies.length === 1) {
            constructedDependency = constructedDependency[0];
        } else {
            let service = DependencyContainer.Instance().resolve(name);
            let dependencies = new Array<any>();

            service.dependencies.forEach(dependency => {
                let d = Container.resolve(dependency);
                dependencies.push(d);
            });

            constructedDependency = service.clazz.construct<TType>(dependencies);
            Container.constructedDependencies.push(constructedDependency);
        }
        return constructedDependency;
    }
}

class Dependency {
    name: string;
    dependencies: Array<string>;
    clazz: Injectable

    constructor(name: string, dependencies: Array<string>, clazz: Injectable) {
        this.name = name;
        this.dependencies = dependencies;
        this.clazz = clazz;
    }
}

class DependencyContainer {
    dependencies: Array<Dependency> = new Array<Dependency>();

    private static currentContainer: DependencyContainer;

    public static Instance = () => {
        if (DependencyContainer.currentContainer) {
            return DependencyContainer.currentContainer;
        }

        return DependencyContainer.currentContainer = new DependencyContainer();
    }

    private constructor() {
    }

    resolve(name: string) {
        let dependencies = this.filterDependencies(name);
        if (dependencies.length === 0) {
            throw new DependencyInjectionException(`The dependency ${name} could not be found`);
        } else if (dependencies.length > 1) {
            throw new DependencyInjectionException(`The dependency ${name} is registred more than once`);
        } else {
            return dependencies[0];
        }
    }

    register(dependency: Dependency) {
        let dependencies = this.filterDependencies(dependency.name);
        if (dependencies.length === 0) {
            this.dependencies.push(dependency);
        } else {
            let dependencyIdx = this.dependencies.indexOf(dependencies[0]);
            this.dependencies[dependencyIdx] = dependency;
        }
    }

    private filterDependencies = (name: string) => {
        return this.dependencies.filter((item) => {
            return item.name === name;
        });
    }
}